import logging

dispatcher = logging.getLogger("aio_connect.dispatcher")
event = logging.getLogger("aio_connect.event")
middlewares = logging.getLogger("aio_connect.middlewares")
webhook = logging.getLogger("aio_connect.webhook")
